unit Ogrille;

interface

uses WinTypes, Controls, Graphics, Classes;

type
    TDrawUser = procedure (Left, Top, Right, Bottom: Integer; Color: TColor) of object;
    TDraw = (CrossLine, Block, Circle, User);
    TGrille = class(TCustomControl)
    private
        { Private-dclarations }
        FTDraw: TDraw;
        FDrawUser: TDrawUser;
        FOnMouseDown: TMouseEvent;
        FOnMouseMove: TMouseMoveEvent;
        FOnMouseUp: TMouseEvent;
        FForeColor: TColor;
        FBackColor: TColor;
        FColCount: Integer;
        FRawCount: Integer;
        procedure DimMemoire;
        procedure SetForeColor(NewColor: TColor);
        procedure SetBackColor(NewColor: TColor);
        procedure SetColcount(NewColCount: Integer);
        procedure SetRawCount(NewRawCount: Integer);
        procedure DrawBack;
        procedure DrawFore;
        procedure DrawTab;
        procedure DrawCell(X, Y: Integer);
        procedure MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
        procedure MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
        procedure MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
        function GetCell(X, Y: Integer): Byte;
        procedure SetCell(X, Y: Integer; const Color: Byte);
        procedure SetTDraw(TypeDraw: TDraw);
    protected
        { Protected-dclarations }
        procedure Paint; override;
    public
        { Public-dclarations }
        Memoire: TMemoryStream;
        Colors: array[1..255] of TColor;
        property Canvas;
        property Cells[X, Y:Integer]: Byte read GetCell write SetCell;
        constructor Create(AOwner: TComponent); override;
        destructor Destroy; override;
        procedure Refresh;
    published
        { Published-dclarations }
        property ForeColor: TColor read FForeColor write SetForeColor default clBlack;
        property BackColor: TColor read FBackColor write SetBackColor default clWhite;
        property ColCount: Integer read FColCount write SetColCount default 10;
        property RawCount: Integer read FRawCount write SetRawCount default 10;
        property Color1: TColor read Colors[1] write Colors[1] default clMaroon;
        property Color2: TColor read Colors[2] write Colors[2] default clGreen;
        property Color3: TColor read Colors[3] write Colors[3] default clOlive;
        property Color4: TColor read Colors[4] write Colors[4] default clNavy;
        property Color5: TColor read Colors[5] write Colors[5] default clPurple;
        property Color6: TColor read Colors[6] write Colors[6] default clTeal;
        property Color7: TColor read Colors[7] write Colors[7] default clGray;
        property Color8: TColor read Colors[8] write Colors[8] default clSilver;
        property Color9: TColor read Colors[9] write Colors[9] default clRed;
        property Color10: TColor read Colors[10] write Colors[10] default clLime;
        property Color11: TColor read Colors[11] write Colors[11] default clBlue;
        property Color12: TColor read Colors[12] write Colors[12] default clFuchsia;
        property Color13: TColor read Colors[13] write Colors[13] default clAqua;
        property Color14: TColor read Colors[14] write Colors[14] default clWhite;
        property Color15: TColor read Colors[15] write Colors[15] default clBlack;
        property AtMouseDown: TMouseEvent read FOnMouseDown write FOnMouseDown;
        property AtMouseMove: TMouseMoveEvent read FOnMouseMove write FOnMouseMove;
        property AtMouseUp: TMouseEvent read FOnMouseUp write FOnMouseUp;
        property AtDrawUser: TDrawUser read FDrawUser write FDrawUser;
        property TypeDraw: TDraw read FTDraw write SetTDraw default Block;
        property Visible;
        property Align;
end;

procedure Register;

implementation


{****************************************************************************}
constructor TGrille.Create(AOwner: TComponent);
var Compteur: Byte;
begin
    inherited Create(AOwner);
    OnMouseDown := MouseDown;
    OnMouseUp := MouseUp;
    OnMouseMove := MouseMove;
    FForeColor := clBlack;
    FBackColor := clWhite;
    FColCount := 10;
    FRawCount := 10;
    Colors[1] := clMaroon;
    Colors[2] := clGreen;
    Colors[3] := clOlive;
    Colors[4] := clNavy;
    Colors[5] :=  clPurple;
    Colors[6] := clTeal;
    Colors[7] := clGray;
    Colors[8] := clSilver;
    Colors[9] := clRed;
    Colors[10] := clLime;
    Colors[11] := clBlue;
    Colors[12] := clFuchsia;
    Colors[13] := clAqua;
    Colors[14] := clWhite;
    Colors[15] := clBlack;
    for Compteur := 16 to 255 do Colors[Compteur] := clWhite;
    Width := 100;
    Height := 100;
    FTDraw := Block;
    Memoire := TMemoryStream.Create;
    DimMemoire;
end;

destructor TGrille.Destroy;
begin
    Memoire.Free;
    inherited Destroy;
end;
{*****************************************************************************}

procedure TGrille.Refresh;
begin
    DrawBack;
    DrawFore;
    DrawTab;
end;

procedure TGrille.Paint;
begin
    Refresh;
end;
{*****************************************************************************}

procedure TGrille.DrawBack;
begin
    Canvas.Brush.Color := FBackColor;
    Canvas.FillRect(Rect(0, 0, Width, Height));
end;

procedure TGrille.DrawFore;
var Compteur: Integer;
    SX, SY: Real;
begin
    Canvas.Pen.Color := FForeColor;
    SX := (Width-1) / FColCount;
    SY := (Height-1) / FRawCount;
    for Compteur := 0 to FColCount-1 do
    begin
        Canvas.MoveTo(Integer(Trunc(Compteur * SX)), 0);
        Canvas.LineTo(Integer(Trunc(Compteur * SX)), Height);
    end;
    Canvas.MoveTo(Width-1, 0);
    Canvas.LineTo(Width-1, Height-1);
    for Compteur := 0 to FRawCount-1 do
    begin
        Canvas.MoveTo(0, Integer(Trunc(Compteur * SY)));
        Canvas.LineTo(Width, Integer(Trunc(Compteur * SY)));
    end;
    Canvas.MoveTo(0, Height-1);
    Canvas.LineTo(Width-1, Height-1);
end;

procedure TGrille.DrawTab;
var Compteur: Integer;
begin
    for Compteur := 0 to (FRawCount*FColCount)-1 do
        DrawCell(Compteur mod FColCount, Compteur div FColCount);
end;

procedure TGrille.DrawCell(X, Y: Integer);
var Color: TColor;
    SX, SY: Real;
    X1, Y1, X2, Y2: Integer;
    Carre: Byte;
begin
    Memoire.Seek(Y*FColCount+X, 0); 
{    Memoire.Seek(Y+X*FRawCount, 0); }
    Memoire.Read(Carre, 1);
    if Carre = 0 then
        Color := FBackColor
    else
        Color := Colors[Carre];
    SX := (Width-1) / FColCount;
    SY := (Height-1) / FRawCount;
    X1 := Integer(Trunc(X*SX));
    Y1 := Integer(Trunc(Y*SY));
    X2 := Integer(Trunc((X+1)*SX));
    Y2 := Integer(Trunc((Y+1)*SY));
    case FTDraw of
        Block:
        begin
            Canvas.Brush.Color := Color;
            Canvas.FillRect(Rect(X1+2, Y1+2, X2-1, Y2-1));
        end;
        CrossLine:
        begin
            Canvas.Pen.Color := Color;
            Canvas.MoveTo(X1+1, Y1+1);
            Canvas.LineTo(X2, Y2);
            Canvas.MoveTo(X1+1, Y2-1);
            Canvas.LineTo(X2, Y1);
        end;
        Circle:
        begin
            Canvas.Pen.Color := FBackColor;
            Canvas.Brush.Color := Color;
            Canvas.Ellipse(X1+2, Y1+2, X2-1, Y2-1);
        end;
        User:
        if Assigned(FDrawUser) then
        begin
            Canvas.Brush.Color := FBackColor;
            FDrawUser(X1, Y1, X2, Y2, Color);
        end;
    end;
end;
{*****************************************************************************}

procedure TGrille.SetForeColor(NewColor: TColor);
begin
    FForeColor := NewColor;
    DrawFore;
end;

procedure TGrille.SetBackColor(NewColor: TColor);
begin
    FBackColor := NewColor;
    Refresh;
end;

procedure TGrille.SetTDraw(TypeDraw: TDraw);
begin
    FTDraw := TypeDraw;
    Refresh;
end;
{*****************************************************************************}

procedure TGrille.SetColcount(NewColCount: Integer);
begin
    if NewColCount > 0 then
    begin
        FColCount := NewColCount;
        DimMemoire;
        Refresh;
    end;
end;

procedure TGrille.SetRawCount(NewRawCount: Integer);
begin
    if NewRawCount > 0 then
    begin
        FRawCount := NewRawCount;
        DimMemoire;
        Refresh;
    end;
end;

procedure TGrille.DimMemoire;
const Vide: Byte = 0;
var Compteur: Integer;
begin
    Memoire.SetSize(FColCount*FRawCount);
    Memoire.Seek(0, 0);
    for Compteur := 1 to FColCount*FRawCount do Memoire.Write(Vide, 1);
end;
{*****************************************************************************}

procedure TGrille.SetCell(X, Y: Integer; const Color: Byte);
begin
    if (X<=FColCount-1) and (Y<=FRawCount-1) and (X>=0) and (Y>=0) then
    begin
        Memoire.Seek(X + Y*FColCount, 0);
        Memoire.Write(Color, 1);
        DrawCell(X, Y);
    end;
end;

function TGrille.GetCell(X, Y: Integer): Byte;
var Ret: Byte;
begin
    if (X<=FColCount) and (Y<=FRawCount) and (X>0) and (Y>0) then
    begin
        Memoire.Seek(X + Y*FColCount, 0);
        Memoire.Read(Ret, 1);
        GetCell := Ret;
    end
    else
        GetCell := 255;
end;
{*******************************************************************************************}

procedure TGrille.MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    if Assigned(FOnMouseDown) then FonMouseDown(Sender, Button, Shift,  Integer(Trunc(X/((Width-1)/FColCount))),
                                                                        Integer(Trunc(Y/((Height-1)/FRawCount))));
end;


procedure TGrille.MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    if Assigned(FOnMouseUp) then FonMouseUp(Sender, Button, Shift,  Integer(Trunc(X/((Width-1)/FColCount))),
                                                                    Integer(Trunc(Y/((Height-1)/FRawCount))));
end;


procedure TGrille.MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
    if Assigned(FOnMouseMove) then
        FonMouseMove(Sender, Shift, Integer(Trunc(X/((Width-1)/FColCount))),
                                    Integer(Trunc(Y/((Height-1)/FRawCount))));
end;

{******************************************************************************************}

procedure Register;
begin
  RegisterComponents('Projet Novell', [TGrille]);
end;

end.
